//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// $Id: $ 
// GEANT4 tag $Name: $
//
//---------------------------------------------------------------------------
//
// ClassName:   G4FTFPAntiBarionBuilder
//
// Author: 2011 J. Apostolakis
//
// Modified:
//
//----------------------------------------------------------------------------
//
#ifndef G4FTFPAntiBarionBuilder_h
#define G4FTFPAntiBarionBuilder_h 1

#include "globals.hh"

#include "G4HadronElasticProcess.hh"
// #include "G4HadronFissionProcess.hh"
// #include "G4HadronCaptureProcess.hh"
#include "G4VAntiBarionBuilder.hh"

#include "G4TheoFSGenerator.hh"
#include "G4ExcitationHandler.hh"
#include "G4PreCompoundModel.hh"
#include "G4GeneratorPrecompoundInterface.hh"
#include "G4FTFModel.hh"
#include "G4LundStringFragmentation.hh"
#include "G4ExcitedStringDecay.hh"
#include "G4QuasiElasticChannel.hh"

#include "G4VCrossSectionDataSet.hh"
#include "G4VComponentCrossSection.hh"

class G4FTFPAntiBarionBuilder : public G4VAntiBarionBuilder
{
  public: 
    G4FTFPAntiBarionBuilder(G4bool quasiElastic=false);
    virtual ~G4FTFPAntiBarionBuilder();

  public: 
    virtual void Build(G4HadronElasticProcess * aP);
    virtual void Build(G4AntiProtonInelasticProcess * aP);
    virtual void Build(G4AntiNeutronInelasticProcess * aP);
    virtual void Build(G4AntiDeuteronInelasticProcess * aP);
    virtual void Build(G4AntiTritonInelasticProcess * aP);
    virtual void Build(G4AntiHe3InelasticProcess * aP);
    virtual void Build(G4AntiAlphaInelasticProcess * aP);
    
    void SetMinEnergy(G4double val) {theMin = val;}
    void SetMaxEnergy(G4double val) {theMax = val;}

  private:
    G4TheoFSGenerator * theModel;
    G4PreCompoundModel * thePreEquilib;
    G4GeneratorPrecompoundInterface * theCascade;
    G4FTFModel * theStringModel;
    G4ExcitedStringDecay * theStringDecay;
    G4QuasiElasticChannel * theQuasiElastic;
    G4LundStringFragmentation * theLund;
    G4ExcitationHandler * theHandler;

    G4VCrossSectionDataSet* theAntiNucleonData;
    G4VComponentCrossSection * theAntiNucleonXS;
    G4double theMin;
    G4double theMax;

};
#endif

